/*                                                                                     
 * Copyright (c) 2018, Intel Corporation                                               
 *                                                                                     
 * Permission is hereby granted, free of charge, to any person obtaining a             
 * copy of this software and associated documentation files (the                       
 * 'Software'), to deal in the Software without restriction, including                 
 * without limitation the rights to use, copy, modify, merge, publish,                 
 * distribute, sublicense, and/or sell copies of the Software, and to                  
 * permit persons to whom the Software is furnished to do so, subject to               
 * the following conditions:                                                           
 *                                                                                     
 * The above copyright notice and this permission notice shall be included             
 * in all copies or substantial portions of the Software.                              
 *                                                                                     
 * THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS             
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                          
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.              
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY                
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,                
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE                   
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                              
*/                                                                                     
                                                                                       
////////////////////////////////////////////////////////////////////////////////       
// !!! WARNING - AUTO GENERATED FILE. DO NOT EDIT DIRECTLY. !!!                        
// Generated by KernelBinToSource.exe tool                                             
////////////////////////////////////////////////////////////////////////////////       
#ifndef __GEN12_HEVC_B_LCU64_H__
#define __GEN12_HEVC_B_LCU64_H__

#include "media_bin_mgr.h"

DECLARE_SHARED_ARRAY_SIZE_UINT32(GEN12_HEVC_B_LCU64_SIZE);
DECLARE_SHARED_ARRAY_UINT32(GEN12_HEVC_B_LCU64);

#if defined(MEDIA_BIN_SUPPORT)
#define GEN12_HEVC_B_LCU64_NAME      "GEN12_HEVC_B_LCU64"

#if defined(MEDIA_BIN_DLL)
static bool registerGEN12_HEVC_B_LCU64 = RegisterMediaBin(GEN12_HEVC_B_LCU64_NAME, GEN12_HEVC_B_LCU64_SIZE, GEN12_HEVC_B_LCU64);
#endif  // defined(MEDIA_BIN_DLL)

#endif  // defined(MEDIA_BIN_SUPPORT)

#endif // __GEN12_HEVC_B_LCU64_H__
